/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: insert.h,v 1.15 2003/05/14 13:58:03 essej Exp $
*/

#ifndef __ardour_insert_h__
#define __ardour_insert_h__

#include <vector>
#include <glib.h>
#include <sigc++/signal_system.h>
#include <string.h>

#include "ardour.h"
#include "redirect.h"

class XMLNode;

namespace LADSPA {
	class Plugin;
}

namespace MIDI {
	class Port;
}

namespace ARDOUR {

class Session;

class Insert : public Redirect
{
  public:
	Insert(Session& s, Redirect::Placement p);
	Insert(Session& s, string name, Redirect::Placement p);
	Insert(Session& s, Redirect::Placement p, int imin, int imax, int omin, int omax);
	
	virtual ~Insert() { }

	virtual void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset) = 0;
	virtual void activate () {}
	virtual void deactivate () {}
};

class PortInsert : public Insert 
{
  public:
	PortInsert (Session&, Redirect::Placement);
	PortInsert (Session&, const XMLNode&);
	PortInsert (const PortInsert&);
	~PortInsert ();

	XMLNode& get_state(void);
	int set_state(const XMLNode&);

	void init ();
	void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);
};

class PluginInsert : public Insert
{
  public:
	PluginInsert (Session&, LADSPA::Plugin&, Redirect::Placement, unsigned int count=1);
	PluginInsert (Session&, const XMLNode&);
	PluginInsert (const PluginInsert&);
	~PluginInsert ();

	XMLNode& get_state(void);
	int set_state(const XMLNode&);
	
	void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);
	void activate ();
	void deactivate ();

	void reset_midi_control (MIDI::Port*, bool);

	void set_control_port (unsigned long port, float val);

	void set_port_automation_playback (unsigned long port, bool yn);
	bool get_port_automation_playback (unsigned long port) const;
	void set_port_automation_record (unsigned long port, bool yn);
	bool get_port_automation_record (unsigned long port) const;

	LADSPA::Plugin& plugin(unsigned int num=0) const
		{
			if (num < _plugins.size()) return *_plugins[num];
			else return *_plugins[0]; // we always have one
		}

	void set_count (unsigned int num);
	unsigned int get_count () { return _plugins.size(); }
	
	string describe_parameter (unsigned long);

  private:

	void control_port_changed (unsigned long, float);
	
	vector<LADSPA::Plugin*> _plugins;
	void automation_run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);
	void connect_and_run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);

	bool* _port_automation_playback;
	bool* _port_automation_record;

	void init ();
};

}; /* namespace ARDOUR */

#endif /* __ardour_insert_h__ */
